//
// Created by chenxiaoyu on 17/2/21.
// Copyright (c) 2017 baidu. All rights reserved.
//

#import <AipBase/AipTokenManager.h>
#import <AipBase/AipOpenUDID.h>
#import "AipOcrService.h"
#import <cepmSdk/cepmSdk.h>

static NSError * ERR_EMPTY_IMAGE;

static NSError * ERR_EMPTY_CREDENTAIL;

static NSError * ERR_IMAGE_UPLOADERR;

static Credential * CREDENTIAL;

static NSString* ErrorDomain = @"com.dhcc.cepm";

@implementation AipOcrService {
    
    AipTokenManager *_aipTokenManager;
    
    NSURLSession *_apiSession;
}

- (instancetype)init {
    if(self = [super init]){
        NSLog(@"AipOcrService 被创建了");
    }
    return self;
}



#pragma mark ocr 接口
/**
 * 身份证正面识别
 * @param image 需要识别的图片
 */
- (IdCardFrontResponse *)detectIdCardFrontFromImage:(UIImage *)image {
    NSLog(@"身份证正面识别");
    OcrClient *ocrClient = [[OcrClient alloc]initWithCredential:CREDENTIAL];
    OcrDataItem *dataItem = [self generateDataItem:image];
    return [ocrClient ocrIdCardFrontWithDataItem:dataItem];
}

/**
 * 身份证背面识别
 * @param image 需要识别的图片
 */
- (IdCardBackResponse *)detectIdCardBackFromImage:(UIImage *)image {
    NSLog(@"身份证背面识别");
    OcrClient *ocrClient = [[OcrClient alloc]initWithCredential:CREDENTIAL];
    OcrDataItem *dataItem = [self generateDataItem:image];
    return [ocrClient ocrIdCardBackWithDataItem:dataItem];
}

/**
 * 银行卡识别
 * @param image 需要识别的图片
 */
- (BankCardResponse *)detectBankCardFromImage:(UIImage *)image {
    NSLog(@"银行卡识别");
    OcrClient *ocrClient = [[OcrClient alloc]initWithCredential:CREDENTIAL];
    OcrDataItem *dataItem = [self generateDataItem:image];
    
    return [ocrClient ocrBankCardWithDataItem:dataItem];
}


- (OcrDataItem *)generateDataItem:(UIImage *)image{
    if(!image){
        @throw ERR_EMPTY_IMAGE;
    }
    
    if(!CREDENTIAL){
        @throw ERR_EMPTY_CREDENTAIL;
    }
    NSString *fileName = @"ocrTemp.jpeg";
    NSString *fileServcerPath = @"";
    BOOL saveFlag = [FileClient saveToLocalWithUiImage:image fileName:fileName];
    if (!saveFlag) {
        @throw ERR_IMAGE_UPLOADERR;
    }
    NSString *filePath = [FileClient getLocalTempFilePath];
    fileServcerPath = [FileClient upload2CmpWithFilePath:filePath fileName:fileName];
    
    OcrDataItem *dataItem = [[OcrDataItem alloc]init];
    dataItem.data = fileServcerPath;
    dataItem.origin = @"fds";
    return dataItem;
}


- (NSData *)jpgDataWithImage:(UIImage *)image sizeLimit:(NSUInteger)maxSize {
    CGFloat compressionQuality = 1.0;
    NSData *imageData = nil;
    
    int i = 0;
    do{
        imageData = UIImageJPEGRepresentation(image, compressionQuality);
        compressionQuality -= 0.1;
        i += 1;
    }while(i < 3 && imageData.length > maxSize);
    return imageData;
}

+ (NSString *)base64Escape:(NSString *)string {
    NSCharacterSet *URLBase64CharacterSet = [[NSCharacterSet characterSetWithCharactersInString:@"/+=\n"] invertedSet];
    return [string stringByAddingPercentEncodingWithAllowedCharacters:URLBase64CharacterSet];
}

+ (NSString *)wwwFormWithDictionary:(NSDictionary *)dict {
    NSMutableString *result = [[NSMutableString alloc] init];
    if (dict != nil) {
        for (NSString *key in dict) {
            if (result.length)
                [result appendString:@"&"];
            [result appendString:[self base64Escape:key]];
            [result appendString:@"="];
            [result appendString:[self base64Escape:dict[key]]];
        }
    }
    return result;
}


#pragma errors

+ (NSError *)aipErrorWithCode:(NSInteger)code andMessage:(NSString *)message {
    NSDictionary *userInfo = @{NSLocalizedDescriptionKey: message};
    return [NSError errorWithDomain:ErrorDomain code:code userInfo:userInfo];
}

+ (NSError *)aipApiServerConnectErrorWithMessage:(NSString *)message {
    NSString *err = [NSString stringWithFormat:@"%@(%@)", @"Fail to connect to api server", message];
    return [AipOcrService aipErrorWithCode:283504 andMessage:err];
}

+ (NSError *)aipApiServerIllegalResponseError {
    return [AipOcrService aipErrorWithCode:283505 andMessage:@"Internal error. Api server responsing illegal data"];
}


+ (instancetype)shardService{
    static AipOcrService *sharedService = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        sharedService = [[self alloc] init];
    });
    ERR_EMPTY_CREDENTAIL = [AipOcrService aipErrorWithCode:283530 andMessage:@"Empty credential. 用户凭证为空"];
    ERR_EMPTY_IMAGE = [AipOcrService aipErrorWithCode:283507 andMessage:@"Empty image. 图片为空"];
    ERR_IMAGE_UPLOADERR = [AipOcrService aipErrorWithCode:283533 andMessage:@"IMAGE_UPLOADERR. 图片上传失败"];
    return sharedService;
}

+(void)setCredential:(Credential *)credential{
    CREDENTIAL = credential;
}

@end
