//
//  FaceClient.swift
//  CepmApp
//
//  Created by 康伟丁 on 2020/12/31.
//

import Foundation
public class FaceClient: NSObject {
    public var credential:Credential?
    
    @objc public init(credential:Credential) {
        self.credential = credential
    }
    
    public func getToken() throws -> String {
        guard self.credential != nil else {throw TokenError.credentialNilError}
        TokenUtils.setCredential(credential: self.credential!)
        let token = try? TokenUtils.getInstance().getToken()
        guard token != nil else {throw TokenError.getTokenError}
        return token!;
    }
    
    
    /// 用于使用视频核身前，获取动作顺序或验证码
    public func validateData(request:ValidateDataRequest) throws -> ValidateDataResponse? {
        let token = try! getToken()
        let cempClient = CepmHttlClient.init(token: token)
        let url = ServiceUrl.baseUrl + ServiceUrl.faceValidateUrl
        let response:ValidateDataResponse? = cempClient.postJson(url, params: request.toJSON())
        return response
    }
    
    /// 照片核身
    public func photoVerify(request:PhotoVerifyRequest) throws -> FaceVerifyResponse? {
        let req = FaceVerifyRequest()
        req.idNum = request.idNum
        req.name = request.name
        
        let dataItem = FaceDataItem()
        dataItem.data = request.data?.data
        dataItem.origin = request.data?.origin ?? DataOrigin.fds.rawValue
        dataItem.imageType = request.imageType
        dataItem.type = FaceType.image.rawValue
        
        req.faceData = dataItem
        
        let token = try! getToken()
        let cempClient = CepmHttlClient.init(token: token)
        let uri = ServiceUrl.baseUrl + ServiceUrl.faceIdNo
        let response:FaceVerifyResponse? = cempClient.postJson(uri, params: req.toJSON())
        return response
    }
    
    
    /// 视频核身
    public func videoVerify(request:VideoVerifyRequest) -> FaceVerifyResponse? {
        let req = FaceVerifyRequest()
        
        let token = try! getToken()
        let uri = ServiceUrl.baseUrl + ServiceUrl.faceIdNo
        
        req.idNum = request.idNum
        req.name = request.name
        req.liveCode = request.liveCode ?? ""
        
        let faceData = FaceDataItem()
        faceData.data = request.data?.data
        faceData.origin = request.data?.origin ?? DataOrigin.selfs.rawValue
        faceData.videoType = request.videoType
        faceData.type = FaceType.video.rawValue
        
        req.faceData = faceData
        
        let cepmClient = CepmHttlClient(token: token)
        let response:FaceVerifyResponse? = cepmClient.postJson(uri, params: req.toJSON())
        return response
    }
    
    /// 照片活体对比
    public func photoVideoVerify(request:PhotoVideoMatchRequest) -> FaceVerifyResponse? {
        let req = FaceMatchRequest()
        
        let token = try! getToken()
        let uri = ServiceUrl.baseUrl + ServiceUrl.faceMatch
        
        req.liveCode = request.liveCode ?? ""
        let faceData = FaceDataItem()
        faceData.data = request.video?.data
        faceData.origin = request.video?.origin ?? DataOrigin.selfs.rawValue
        faceData.videoType = request.videoType
        faceData.type = FaceType.video.rawValue
        req.faceData = faceData
        
        let photo = FaceDataItem()
        photo.data = request.photo?.data
        photo.origin = request.photo?.origin ?? DataOrigin.selfs.rawValue
        photo.type = FaceType.image.rawValue
        req.image = photo
        
        let cepmClient = CepmHttlClient(token: token)
        let response:FaceVerifyResponse? = cepmClient.postJson(uri, params: req.toJSON())
        return response
    }
    
    /// 照片对比
    public func photoCompare(request: PhotoCompareRequest) -> FaceVerifyResponse? {
        let req = FaceMatchRequest()
        
        let token = try! getToken()
        let uri = ServiceUrl.baseUrl + ServiceUrl.faceMatch
        
        let faceData = FaceDataItem()
        faceData.data = request.photo?.data
        faceData.origin = request.photo?.origin ?? DataOrigin.selfs.rawValue
        faceData.type = FaceType.image.rawValue
        req.faceData = faceData
        
        let photo = FaceDataItem()
        photo.data = request.originPhoto?.data
        photo.origin = request.originPhoto?.origin ?? DataOrigin.selfs.rawValue
        photo.type = FaceType.image.rawValue
        req.image = photo
        
        let cepmClient = CepmHttlClient(token: token)
        let response:FaceVerifyResponse? = cepmClient.postJson(uri, params: req.toJSON())
        return response
    }
    
    /// 离线采集后使用该“照片比对”接口
    @objc public func sdkPhotoCompare(request: PhotoCompareRequest) -> FaceVerifyResponse? {
        let req = SdkFaceMatchRequest()
        
        let token = try! getToken()
        credential?.url = (credential?.url) ?? ServiceUrl.baseUrl
        let uri = (credential?.url)! + ServiceUrl.sdkFaceMatch
        
        let faceData = FaceDataItem()
        faceData.data = request.photo?.data
        faceData.origin = request.photo?.origin ?? DataOrigin.selfs.rawValue
        faceData.type = FaceType.image.rawValue
        req.photo = faceData
        
        let photo = FaceDataItem()
        photo.data = request.originPhoto?.data
        photo.origin = request.originPhoto?.origin ?? DataOrigin.selfs.rawValue
        photo.type = FaceType.image.rawValue
        req.originPhoto = photo
        
        let cepmClient = CepmHttlClient(token: token)
        let response:FaceVerifyResponse? = cepmClient.postJson(uri, params: req.toJSON())
        return response
    }
    
    /// 离线采集后使用该“照片核身”接口
    @objc public func sdkPhotoVerify(request: PhotoVerifyRequest) -> FaceVerifyResponse? {
        let req = SdkFaceVerifyRequest()
        
        let token = try! getToken()
        credential?.url = (credential?.url) ?? ServiceUrl.baseUrl
        let uri = (credential?.url)! + ServiceUrl.sdkFaceVerify
        
        req.idNum = request.idNum
        req.name = request.name
        
        let faceData = FaceDataItem()
        faceData.data = request.data?.data
        faceData.origin = request.data?.origin ?? DataOrigin.selfs.rawValue
        faceData.imageType = request.imageType
        faceData.type = FaceType.image.rawValue
        
        req.photo = faceData
        
        let cepmClient = CepmHttlClient(token: token)
        let response:FaceVerifyResponse? = cepmClient.postJson(uri, params: req.toJSON())
        return response
    }
}
