//
//  FileClient.swift
//  CepmApp
//
//  Created by 康伟丁 on 2020/12/31.
//

import Foundation
import UIKit
public class FileClient: NSObject {
    
    /// 文件上传并压缩，此方法上传的文件只在服务器保留一天
    /// - Parameters:
    ///   - filePath: 文件路径
    ///   - fileName: 文件名称
    /// - Returns: 若请求成功返回文件所在url，若失败则返回空字符串
    @objc public class func upload2Cmp(filePath: String, fileName: String) -> String {
        var path = filePath
        if !path.hasSuffix("/"){
            path = path + "/"
        }
        let response:FileResponse? = HttpUtils.doUploadFileSync(ServiceUrl.uploadCmp, filePath: path, fileName: fileName)
        guard response != nil else {
            return ""
        }
        guard response?.errcode == 0 else {
            return ""
        }
        return (response?.data)!
    }
    
    /// 文件上传
    /// - Parameters:
    ///   - filePath: 文件路径
    ///   - fileName: 文件名称
    /// - Returns: 若请求成功返回文件所在url，若失败则返回空字符串
    @objc public class func upload(filePath: String, fileName: String) -> String {
        var path = filePath
        if !path.hasSuffix("/"){
            path = path + "/"
        }
        let response:FileResponse? = HttpUtils.doUploadFileSync(ServiceUrl.upload, filePath: filePath, fileName: fileName)
        guard response != nil else {
            return ""
        }
        guard response!.errcode == 0 else {
            return ""
        }
        return (response!.data)!
    }
    
    /// 将临时图片保存到本地
    /// - Parameter uiImage: UIImage
    /// - Parameter fileName: 文件名称，包含后缀，后缀请小写，例如123.jpeg
    /// - Returns: 文件名称
    @objc public class func saveToLocal(uiImage:UIImage, fileName:String) -> Bool {
        let imageData:Data? = uiImage.jpegData(compressionQuality: 1.0)
        let localTempPath = getLocalTempFilePath() +  fileName
        let url = URL.init(fileURLWithPath: localTempPath)
        do {
            try imageData?.write(to: url, options: .atomic)
        }catch {
            return false
        }
        return true
    }
    
    @objc public class func getLocalTempFilePath() -> String {
        let tempDir = NSTemporaryDirectory()
        return tempDir
    }
}
