//
//  OrcClient.swift
//  CepmApp
//
//  Created by 楚海 on 2020/12/30.
//

import Foundation

public class OcrClient:NSObject{
    private var credential:Credential
    
    @objc public init(credential:Credential) {
        self.credential = credential
    }
    
    private func getToken() throws -> String{
        TokenUtils.setCredential(credential: credential)
        guard let token:String = try? TokenUtils.getInstance().getToken() else {
            throw TokenError.getTokenError
        }
        return token
    }
    
    /// 身份证背面识别
    /// - Parameter dataItem: 数据信息
    /// - Returns: 识别结果实体
    @objc public func ocrIdCardBack(dataItem:OcrDataItem) -> IdCardBackResponse?{
        return ocr(dataItem: dataItem, ocrType: OcrType.idcard_back)
    }
    
    /// 身份证正面识别
    /// - Parameter dataItem: 数据信息
    /// - Returns: 识别结果实体
    @objc public func ocrIdCardFront(dataItem:OcrDataItem) -> IdCardFrontResponse?{
        return ocr(dataItem: dataItem, ocrType: OcrType.idcard_front)
    }
    
    /// 银行卡识别
    /// - Parameter dataItem: 数据信息
    /// - Returns: 识别结果实体
    @objc public func ocrBankCard(dataItem:OcrDataItem) -> BankCardResponse?{
        return ocr(dataItem: dataItem, ocrType: OcrType.creditcard)
    }
    
    private func ocr<T:BaseEntry>(dataItem:OcrDataItem,ocrType:OcrType) -> T?{
        var result : T?
        do{
            let token = try getToken()
            let hc = CepmHttlClient(token: token)
            let request = OcrRequest()
            dataItem.type = ocrType.rawValue
            request.image = dataItem
            result = hc.postJson(credential.url! + ServiceUrl.ocrUrl, params: request.toJSON())
        }catch {
            print(error)
        }
        return result
    }
}
