//
//  TokenApiCepm.swift
//  CepmApp
//
//  Created by 楚海 on 2020/12/22.
//

import Foundation
import Alamofire


public class SysTokenApi: TokenApi {
    
    /// 私有构造函数
    private init(){
        
    }
    
    /// 用户凭据
    private var credential:Credential?
    
    /// 单例实体
    private static let shared : SysTokenApi = SysTokenApi()
    
    /// 获取单例对对象
    /// - Parameter credential: 用户凭证
    /// - Returns:  单例对象
    public class func getInstance(_ credential:Credential) -> SysTokenApi{
        if shared.credential == nil  {
            shared.credential = credential;
        }
        return shared;
    }
    
    /// 获取单例对对象
    /// - Returns:  单例对象
    public class func getInstance() -> SysTokenApi{
        return shared;
    }
    
    
    
    /// 获取虚拟用户Token
    public func getUserToken() throws -> Token? {
        if credential == nil{
            throw TokenError.credentialNilError
        }
        let url = (credential!.url!) + TokenConstants.getTokenUrl
        let params = credential!.toJSON()
        
        return HttpUtils.doPostSyn(url, params: params)
    }
    
    /// 刷新token
    /// - Parameter refreshToken: 刷新token的token
    public func refreshUserToken(refreshToken: String) throws -> Token? {
        if credential == nil{
            throw TokenError.credentialNilError
        }
        let url = (credential!.url!) + TokenConstants.refreshTokenUrl
        var params = [String:Any]()
        params["refreshToken"] = refreshToken
        
        return HttpUtils.doPostSyn(url, params: params)
    }
    
    
}
