//
//  TokenUtils.swift
//  CepmApp
//
//  Created by 楚海 on 2020/12/25.
//

import Foundation

public class TokenUtils{
    
    /// 单例对象
    private static let shared = TokenUtils()
    
    /// Token内容字典
    private static var TOKEN_CONTENT = [String:Any]()
    
    /// token
    static let ACCESS_TOKEN = "accessToken";
    
    /// 刷新Token的Token
    static let REFRESH_TOKEN = "refreshToken";
    
    /// 获取Token的时间
    static let GEN_TIME = "genTime";
    
    /// 错误编号
    static let ERROR_CODE = "errCode";
    
    /// 错误信息
    static let ERROR_MESSAGE = "errMsg";
    
    /// tokenApi对象
    static var tokenApi:TokenApi?
    
    /// 用户凭证对象
    static var credential:Credential?
    
    /// 私有构造函数，配合单例
    private init(){
        
    }
    
    /// 获取单例对象
    /// - Returns: 单例对象
    class func getInstance() -> TokenUtils{
        tokenApi = SysTokenApi.getInstance(credential!)
        return shared
    }
    
    /// 设置用户凭证
    /// - Parameter credential: 用户凭证对象
    class func setCredential(credential:Credential){
        TokenUtils.credential = credential
    }
    
    /// 从服务器获取Token
    /// - Throws: 获取token为nil抛出TokenError.getTokenError错误
    private func getTokenFromServer() throws {
        guard let token = try TokenUtils.tokenApi!.getUserToken() else {
            throw TokenError.getTokenError
        }
        TokenUtils.TOKEN_CONTENT.removeAll()
        if HttpConstants.successCode == token.errcode!{
            TokenUtils.TOKEN_CONTENT[TokenUtils.ACCESS_TOKEN] = token.accessToken!
            TokenUtils.TOKEN_CONTENT[TokenUtils.REFRESH_TOKEN] = token.refreshToken == nil ? "" : token.refreshToken!
            TokenUtils.TOKEN_CONTENT[TokenUtils.GEN_TIME] = Date.currentDate
        }else{
            TokenUtils.TOKEN_CONTENT[TokenUtils.ERROR_CODE] = token.errcode!
            TokenUtils.TOKEN_CONTENT[TokenUtils.ERROR_MESSAGE] = token.errmsg == nil ? "":  token.errmsg!
        }
    }
    
    /// 从服务器刷新Token
    /// - Throws: 刷新后的token为nil抛出TokenError.refreshTokenError错误
    private func refreshTokenFromServer() throws {
        let refreshToken = TokenUtils.TOKEN_CONTENT[TokenUtils.REFRESH_TOKEN] as! String
        if "" == refreshToken{
            try getTokenFromServer()
        }else{
            guard let token = try TokenUtils.tokenApi!.refreshUserToken(refreshToken: refreshToken) else {
                throw TokenError.refreshTokenError
            }
            if HttpConstants.successCode == token.errcode!{
                TokenUtils.TOKEN_CONTENT[TokenUtils.ACCESS_TOKEN] = token.accessToken!
            }else{
                TokenUtils.TOKEN_CONTENT.removeAll()
                TokenUtils.TOKEN_CONTENT[TokenUtils.ERROR_CODE] = token.errcode!
                TokenUtils.TOKEN_CONTENT[TokenUtils.ERROR_MESSAGE] = token.errmsg == nil ? "":  token.errmsg!
            }
        }
    }
    
    
    /// 检查refreshToken是否过期
    /// - Returns: true：已经过期
    private func checkRefreshTokenExpire() -> Bool{
        if TokenUtils.TOKEN_CONTENT.isEmpty {
            return true
        }
        guard let genTime = TokenUtils.TOKEN_CONTENT[TokenUtils.GEN_TIME] as? Date else {
            return true;
        }
        
        return genTime.compareOffset(1, dateType: .hours) ==  1
    }
    
    
    /// 检查token是否过期
    /// - Returns: true：已经过期
    private func checkTokenExpire() -> Bool {
        if checkRefreshTokenExpire() {
            return true
        }
        
        guard let genTime = TokenUtils.TOKEN_CONTENT[TokenUtils.GEN_TIME] as? Date else {
            return true;
        }
        
        return genTime.compareOffset(14, dateType: .mimute) ==  1
    }
    
    /// 获取Token
    /// - Throws: 获取token时候发生的错误
    /// - Returns: token串
    func getToken() throws -> String {
        if(checkTokenExpire()){
            if(checkRefreshTokenExpire()){
                try self.getTokenFromServer()
            }else{
                try self.refreshTokenFromServer()
            }
        }
        
        guard let token = TokenUtils.TOKEN_CONTENT[TokenUtils.ACCESS_TOKEN] as? String else{
            throw TokenError.getTokenError
        }
        
        return token;
    }
    
}
