//
//  TtsClient.swift
//  CepmApp
//
//  Created by 康伟丁 on 2020/12/29.
//

import Foundation
import AVFoundation

public class DhccTtsClient: NSObject {
    
    public var credential:Credential?
    
    
    public init(credential:Credential) {
        self.credential = credential
    }
    
    /// 语音合成业务方法
    /// - Parameter request: TtsRequest请求封装实体
    /// - Throws: TokenError.getTokenError、TokenError.credentialNilError
    /// - Returns: TtsResponse业务实体
    @objc public func doTts(request:TtsRequest) -> TtsResponse {
//        guard self.credential != nil else {throw TokenError.credentialNilError}
        TokenUtils.setCredential(credential: self.credential!)
        let token = try? TokenUtils.getInstance().getToken()
//        guard token != nil else {throw TokenError.getTokenError}
        let cempClient = CepmHttlClient.init(token: token!)
        let uri = credential!.url! + ServiceUrl.ttsUrl
        let response:TtsResponse? = cempClient.postJson(uri, params: request.toJSON())
        return response!
    }
    
    
    /// 播放方法
    /// - Parameter response: doTts返回的TtsResponse
    /// - Returns: AVAudioPlayer播放控件
    @objc public func play(response:TtsResponse) -> AVAudioPlayer {
        let base64:String = (response.data?.result?.text)!
        let audioData = Data(base64Encoded: base64)
        if audioData != nil {
            let filename = getURLforMemo()
            do {
                try audioData!.write(to: filename, options: .atomicWrite)
                do {
                    let audioPlayer:AVAudioPlayer = try AVAudioPlayer.init(contentsOf: filename)
                    return audioPlayer
                } catch let error {
                    print(error.localizedDescription)
                }
            } catch let err {
                print(err.localizedDescription)
            }
        }
        return AVAudioPlayer()
    }
    

    /// 获取缓存临时目录URL
    /// - Returns: URL
    public func getURLforMemo() -> URL {
        let tempDir = NSTemporaryDirectory()
        let filePath = tempDir + "temp.wav"
        return URL.init(fileURLWithPath: filePath)
    }
    
}
